module Restcomm
  module REST
    class BaseClient
      include Restcomm::Util
      include Restcomm::REST::Utils
require 'active_support/all'


      HTTP_HEADERS = {
          'Accept' => 'application/json',
          'Accept-Charset' => 'utf-8',
          'User-Agent' => "restcomm-ruby/#{Restcomm::VERSION}" \
                        " (#{RUBY_ENGINE}/#{RUBY_PLATFORM}" \
                        " #{RUBY_VERSION}-p#{RUBY_PATCHLEVEL})"
      }



      attr_reader :account_sid, :last_request, :last_response





      def initialize(*args)
        options = args.last.is_a?(Hash) ? args.pop : {host: args[2]}
        @config = get_defaults.merge! options

        @account_sid = args[0] || Restcomm.account_sid
        @auth_token = args[1] || Restcomm.auth_token
	@host=args[2]


        if @account_sid.nil? || @auth_token.nil?
          raise ArgumentError, 'Account SID and auth token are required'
        end

        if @host.nil?
          raise ArgumentError, 'Destination Host IP cannot is required!'
        end

        set_up_connection
        set_up_subresources

         end


 DEFAULTS = {
          host: '',
          port: 8080,
          use_ssl: false,
          ssl_verify_peer: false,
          ssl_ca_file: File.dirname(__FILE__) + '/../../../conf/cacert.pem',
          timeout: 30,
          proxy_addr: nil,
          proxy_port: nil,
          proxy_user: nil,
          proxy_pass: nil,
          retry_limit: 1
      }


      protected

      ##
      # Get the default config values.

      def get_defaults
        # To be overridden
	DEFAULTS
      end

      ##
      # Set up and cache a Net::HTTP object to use when making requests. This is
      # a private method documented for completeness.
      def set_up_connection # :doc:
        connection_class = Net::HTTP::Proxy @config[:proxy_addr],
                                            @config[:proxy_port], @config[:proxy_user], @config[:proxy_pass]
        @connection = connection_class.new @config[:host], @config[:port]
        set_up_ssl
        @connection.open_timeout = @config[:timeout]
        @connection.read_timeout = @config[:timeout]
      end

      ##
      # Set up the ssl properties of the <tt>@connection</tt> Net::HTTP object.
      # This is a private method documented for completeness.
      def set_up_ssl # :doc:
        @connection.use_ssl = @config[:use_ssl]
        if @config[:ssl_verify_peer]
          @connection.verify_mode = OpenSSL::SSL::VERIFY_PEER
          @connection.ca_file = @config[:ssl_ca_file]
        else
          @connection.verify_mode = OpenSSL::SSL::VERIFY_NONE
        end
      end

      ##
      # Set up sub resources attributes.
      def set_up_subresources # :doc:
        # To be overridden
      end

      ##
      # Send an HTTP request using the cached <tt>@connection</tt> object and
      # return the JSON response body parsed into a hash. Also save the raw
      # Net::HTTP::Request and Net::HTTP::Response objects as
      # <tt>@last_request</tt> and <tt>@last_response</tt> to allow for
      # inspection later.
      def connect_and_send(request) # :doc:
        @last_request = request
        retries_left = @config[:retry_limit]
        begin
          response = @connection.request request
          @last_response = response
          if response.kind_of? Net::HTTPServerError
            raise Restcomm::REST::ServerError
          end
        rescue Exception
          raise if request.class == Net::HTTP::Post
          if retries_left > 0 then retries_left -= 1; retry else raise end
        end

        if response.body and !response.body.empty?


		if response.body.include? ("<RestcommResponse>")
			m = Hash.from_xml(response.body).to_json 
			object = MultiJson.load(m)
		else
		object = MultiJson.load(response.body)
		end
        end

        if response.kind_of? Net::HTTPClientError
          raise Restcomm::REST::RequestError.new object['message'], object['code']

        end
        object
      end
    end

    ##
    # The Restcomm::REST::Client class caches authentication parameters and
    # exposes methods to make HTTP requests to Restcomm's REST API. However, you
    # should never really need to call these methods yourself since you can
    # work with the more pleasant wrapper objects like Restcomm::REST::Call.
    #
    # Instantiate a client like so:
    #
    #   @client = Restcomm::REST::Client.new account_sid, auth_token
    #
    # There are a few options you can use to configure the way your client will
    # communicate with Restcomm. See #new for a list and descriptions.
    #
    # Once you have a client object you can use it to do fun things. Every
    # client object exposes two wrapper objects which you can use as entry
    # points into Restcomm: +account+ and +accounts+.
    #
    # ==== @client.account
    #
    # Most of the time you'll want to start with the +account+ attribute. This
    # object is an instance of Restcomm::REST::Account that wraps the account
    # referenced by the +account_sid+ you used when instantiating the client.
    #
    # An instance of Restcomm::REST::Account exposes objects wrapping all of the
    # account-level Restcomm resources as properties. So
    #
    #   @client.account.calls
    #
    # For convenience, the resources of the default account are also available
    # on the client object. So the following call is equivalent to the example
    # above
    #
    #   @client.calls
    #
    # represents an account's call list.
    #
    # ==== @client.accounts
    #
    # If you are doing anything related to subaccounts you'll want to start
    # here. This object is an instance of Restcomm::REST::Accounts that wraps
    # the list of accounts belonging to the master account referenced by
    # the +account_sid+ used to instantiate the client.
    #
    # This class inherits from Restcomm::REST::ListResource, so you can use
    # methods like ListResource#list to return a (possibly filtered) list of
    # accounts and ListResource#create to create a new account. Use
    # ListResource#get to grab a particular account once you know its sid.
    class Client < BaseClient
      API_VERSION = '2012-04-24'

      attr_reader :account, :accounts

      ##
      # Instantiate a new HTTP client to talk to Restcomm. The parameters
      # +account_sid+ and +auth_token+ are required, unless you have configured
      # them already using the block configure syntax, and used to generate the
      # HTTP basic auth header in each request. The +options+ parameter is a
      # hash of connection configuration options. the following keys are
      # supported:
      #
      # === <tt>host: 'api.restcomm.com'</tt>
      #
      # The domain to which you'd like the client to make HTTP requests. Useful
      # for testing. Defaults to 'api.restcomm.com'.
      #
      # === <tt>port: 443</tt>
      #
      # The port on which to connect to the above domain. Defaults to 443 and
      # should be left that way except in testing environments.
      #
      # === <tt>use_ssl: true</tt>
      #
      # Declare whether ssl should be used for connections to the above domain.
      # Defaults to true and should be left alone except when testing.
      #
      # === <tt>ssl_verify_peer: true</tt>
      #
      # Declare whether to verify the host's ssl cert when setting up the
      # connection to the above domain. Defaults to true, but can be turned off
      # to avoid ssl certificate verification failures in environments without
      # the necessary ca certificates.
      #
      # === <tt>ssl_ca_file: '/path/to/ca/file'</tt>
      #
      # Specify the path to the certificate authority bundle you'd like to use
      # to verify Restcomm's SSL certificate on each request. If not specified, a
      # certificate bundle extraced from Firefox is packaged with the gem and
      # used by default.
      #
      # === <tt>timeout: 30</tt>
      #
      # Set the time in seconds to wait before timing out the HTTP request.
      # Defaults to 30 seconds. If you aren't fetching giant pages of call or
      # SMS logs you can safely decrease this to something like 3 seconds or
      # lower. In paricular if you are sending SMS you can set this to 1 second
      # or less and swallow the exception if you don't care about the response.
      #
      # === <tt>proxy_addr: 'proxy.host.domain'</tt>
      #
      # The domain of a proxy through which you'd like the client to make HTTP
      # requests. Defaults to nil.
      #
      # === <tt>proxy_port: 3128</tt>
      #
      # The port on which to connect to the above proxy. Defaults to nil.
      #
      # === <tt>proxy_user: 'username'</tt>
      #
      # The user name to use for authentication with the proxy. Defaults to nil.
      # 
      # === <tt>proxy_pass: 'password'</tt>
      #
      # The password to use for authentication with the proxy. Defaults to nil.
      #
      # === <tt>retry_limit: 1</tt>
      #
      # The number of times to retry a request that has failed before throwing
      # an exception. Defaults to one.
      def initialize(*args)
        super(*args)
      end

      def inspect # :nodoc:
        "<Restcomm::REST::Client @account_sid=#{@account_sid}>"
      end

      ##
      # Define #get, #put, #post and #delete helper methods for sending HTTP
      # requests to Restcomm. You shouldn't need to use these methods directly,
      # but they can be useful for debugging. Each method returns a hash
      # obtained from parsing the JSON object in the response body.
      [:get, :put, :post, :delete].each do |method|
        method_class = Net::HTTP.const_get method.to_s.capitalize
        define_method method do |path, *args|
          params = restify args[0]; params = {} if params.empty?
          unless args[1] # build the full path unless already given
         path = "/restcomm#{path}.json"

	if method == :get

		if  path.include?("/Accounts/#{@account_sid}.json" )
				path.gsub!("/Accounts/#{@account_sid}.json", "/Accounts.json/#{@account_sid}")
	 	end
		

		if   path.match(/Calls\/CA.*\/Recordings.json/ )		
			path.gsub!("/Recordings.json", ".json")
	 	end

		if  path.include?("/Accounts/#{@account_sid}/AvailablePhoneNumbers/US/Local.json" )
				path.gsub!("/AvailablePhoneNumbers/US/Local.json", "/AvailablePhoneNumbers/US/Local")
	 	end

		if  path.include?("/Accounts/#{@account_sid}.json" )
				path.gsub!("/Accounts/#{@account_sid}.json", "/Accounts.json/#{@account_sid}")
	 	end
	end


#### Make Restcomm path compatible with Twilio -  convert Restcomm 2012-04-24/Accounts/ACae6e420f425248d6a26948c17a9e2acf/Calls/CA76d50db1b78d4a34bfaa708bc669d7db.json to /2012-04-24/Accounts/ACae6e420f425248d6a26948c17a9e2acf/Calls.json/CA76d50db1b78d4a34bfaa708bc669d7db 

	if method == :post

		if   path.match(/Calls\/CA.*/ )
			call_sid = path.split("/").last	
			call_sid = call_sid.gsub!(".json", "")	
			path.gsub!("Calls/#{call_sid}.json", "Calls.json/#{call_sid}")
			
			
	 	end
	end	

	 path << "?#{url_encode(params)}" if method == :get && !params.empty?
          end
          request = method_class.new path, HTTP_HEADERS
          request.basic_auth @account_sid, @auth_token
          request.form_data = params if [:post, :put].include? method
          connect_and_send request
        end
      end

      ##
      # Delegate account methods from the client. This saves having to call
      # <tt>client.account</tt> every time for resources on the default
      # account.
      def method_missing(method_name, *args, &block)
        if account.respond_to?(method_name)
          account.send(method_name, *args, &block)
        else
          super
        end
      end

      def respond_to?(method_name, include_private=false)
        if account.respond_to?(method_name, include_private)
          true
        else
          super
        end
      end

      protected

      ##
      # Set up +account+ and +accounts+ attributes.
      def set_up_subresources # :doc:
        @accounts = Restcomm::REST::Accounts.new "/#{API_VERSION}/Accounts", self
        @account = @accounts.get @account_sid
      end
    end

    class TaskRouterClient < BaseClient
      API_VERSION = 'v1'

      DEFAULTS = {
          host: '192.168.1.8',
          port: 80,
          use_ssl: false,
          ssl_verify_peer: false,
          ssl_ca_file: File.dirname(__FILE__) + '/../../../conf/cacert.pem',
          timeout: 30,
          proxy_addr: nil,
          proxy_port: nil,
          proxy_user: nil,
          proxy_pass: nil,
          retry_limit: 1
      }

      attr_reader :workspace, :workspace_sid, :workspaces

      ##
      # Instantiate a new HTTP TaskRouter client to talk to Restcomm. The parameters
      # +account_sid+, +auth_token+ and +workspace_sid are required, unless you
      # have configured them already using the block configure syntax, and used
      # to generate the HTTP basic auth header in each request. The +options+
      # parameter is a hash of connection configuration options. the following
      # keys are supported:
      #
      # === <tt>host: 'taskrouter.restcomm.com'</tt>
      #
      # The domain to which you'd like the client to make HTTP requests. Useful
      # for testing. Defaults to 'api.restcomm.com'.
      #
      # === <tt>port: 443</tt>
      #
      # The port on which to connect to the above domain. Defaults to 443 and
      # should be left that way except in testing environments.
      #
      # === <tt>use_ssl: true</tt>
      #
      # Declare whether ssl should be used for connections to the above domain.
      # Defaults to true and should be left alone except when testing.
      #
      # === <tt>ssl_verify_peer: true</tt>
      #
      # Declare whether to verify the host's ssl cert when setting up the
      # connection to the above domain. Defaults to true, but can be turned off
      # to avoid ssl certificate verification failures in environments without
      # the necessary ca certificates.
      #
      # === <tt>ssl_ca_file: '/path/to/ca/file'</tt>
      #
      # Specify the path to the certificate authority bundle you'd like to use
      # to verify Restcomm's SSL certificate on each request. If not specified, a
      # certificate bundle extraced from Firefox is packaged with the gem and
      # used by default.
      #
      # === <tt>timeout: 30</tt>
      #
      # Set the time in seconds to wait before timing out the HTTP request.
      # Defaults to 30 seconds. If you aren't fetching giant pages of call or
      # SMS logs you can safely decrease this to something like 3 seconds or
      # lower. In paricular if you are sending SMS you can set this to 1 second
      # or less and swallow the exception if you don't care about the response.
      #
      # === <tt>proxy_addr: 'proxy.host.domain'</tt>
      #
      # The domain of a proxy through which you'd like the client to make HTTP
      # requests. Defaults to nil.
      #
      # === <tt>proxy_port: 3128</tt>
      #
      # The port on which to connect to the above proxy. Defaults to nil.
      #
      # === <tt>proxy_user: 'username'</tt>
      #
      # The user name to use for authentication with the proxy. Defaults to nil.
      #
      # === <tt>proxy_pass: 'password'</tt>
      #
      # The password to use for authentication with the proxy. Defaults to nil.
      #
      # === <tt>retry_limit: 1</tt>
      #
      # The number of times to retry a request that has failed before throwing
      # an exception. Defaults to one.
      def initialize(*args)
        @workspace_sid = args[2]
        if @workspace_sid.nil?
          raise ArgumentError, 'Workspace SID is required'
        end
        super(*args)
      end

      ##
      # Define #get, #put, #post and #delete helper methods for sending HTTP
      # requests to Restcomm. You shouldn't need to use these methods directly,
      # but they can be useful for debugging. Each method returns a hash
      # obtained from parsing the JSON object in the response body.
      [:get, :put, :post, :delete].each do |method|
        method_class = Net::HTTP.const_get method.to_s.capitalize
        define_method method do |path, *args|
          params = restify args[0]; params = {} if params.empty?
          unless args[1] # build the full path unless already given
            path = path.dup
            path << "?#{url_encode(params)}" if method == :get && !params.empty?

          end
          request = method_class.new path, HTTP_HEADERS
          request.basic_auth @account_sid, @auth_token
          request.form_data = params if [:post, :put].include? method
          connect_and_send request
        end
      end

      def inspect # :nodoc:
        "<Restcomm::REST::TaskRouterClient @account_sid=#{@account_sid}>"
      end

      ##
      # Delegate workspace methods from the client. This saves having to call
      # <tt>client.workspace</tt> every time for resources on the default
      # workspace.
      def method_missing(method_name, *args, &block)
        if workspace.respond_to?(method_name)
          workspace.send(method_name, *args, &block)
        else
          super
        end
      end

      def respond_to?(method_name, include_private=false)
        if workspace.respond_to?(method_name, include_private)
          true
        else
          super
        end
      end

      ##
      # Get statistics of a task queue.
      def task_queue_statistics(task_queue_sid, *args) # :doc:
        if task_queue_sid.nil?
          raise ArgumentError, 'Task queue SID is required'
        end
        path = "/#{API_VERSION}/Workspaces/#{@workspace_sid}/TaskQueues/#{task_queue_sid}/Statistics"
        response = get path, args, true
        Restcomm::REST::TaskRouter::TaskQueueStatistics.new path, self, response
      end

      ##
      # Get statistics of task queues.
      def task_queues_statistics(*args) # :doc:
        path = "/#{API_VERSION}/Workspaces/#{@workspace_sid}/TaskQueues/Statistics"
        stats = Restcomm::REST::TaskRouter::TaskQueuesStatistics.new path, self
        stats.list args, true
      end

      ##
      # Get statistics of a worker.
      def worker_statistics(worker_sid, *args) # :doc:
        if worker_sid.nil?
          raise ArgumentError, 'Worker SID is required'
        end
        path = "/#{API_VERSION}/Workspaces/#{@workspace_sid}/Workers/#{worker_sid}/Statistics"
        response = get path, args, true
        Restcomm::REST::TaskRouter::WorkerStatistics.new path, self, response
      end

      ##
      # Get statistics of workers.
      def workers_statistics(*args) # :doc:
        path = "/#{API_VERSION}/Workspaces/#{@workspace_sid}/Workers/Statistics"
        response = get path, args, true
        Restcomm::REST::TaskRouter::WorkersStatistics.new path, self, response
      end

      ##
      # Get statistics of a workflow.
      def workflow_statistics(workflow_sid, *args) # :doc:
        if workflow_sid.nil?
          raise ArgumentError, 'Workflow SID is required'
        end
        path = "/#{API_VERSION}/Workspaces/#{@workspace_sid}/Workflows/#{workflow_sid}/Statistics"
        response = get path, args, true
        Restcomm::REST::TaskRouter::WorkflowStatistics.new path, self, response
      end

      ##
      # Get statistics of a workspace.
      def workspace_statistics(*args) # :doc:
        path = "/#{API_VERSION}/Workspaces/#{@workspace_sid}/Statistics"
        response = get path, args, true
        Restcomm::REST::TaskRouter::WorkspaceStatistics.new path, self, response
      end

      protected

      ##
      # Get the default config values.
      def get_defaults
        DEFAULTS
      end

      ##
      # Set up +workspace+ and +workspaces+ attributes.
      def set_up_subresources # :doc:
        @workspaces = Restcomm::REST::TaskRouter::Workspaces.new "/#{API_VERSION}/Workspaces", self
        @workspace = @workspaces.get @workspace_sid
      end

    end
  end
end
